#include "stdafx.h"
#include <memory.h>
#include "AvSyncDecodeWrapper.h"


// Function     : CAvSyncDecodeWrapper::CAvSyncDecodeWrapper
// Description  : Constructor
// Parameter    :
//              | [in] pfDecodeFrame        @DecodeFrame callback function
//              | [in] dwDecodeFrameContext @DecodeFrame context
// Return value : N/A
//
CAvSyncDecodeWrapper::CAvSyncDecodeWrapper(LPDECODEFRAMECALLBACK pfDecodeFrame, DWORD dwDecodeFrameContext)
{
	m_hAvSync = NULL;
	m_hDecodeChannel = NULL;

	m_dwDecodeFormat = PF_YUV;  // Initialize the decode format as YUV

	memset(&m_tParam, 0, sizeof(m_tParam));

	m_tParam.pfDecodeFrame = pfDecodeFrame;

	m_tParam.dwDecodeFrameContext = dwDecodeFrameContext;

	InitialModule();  // Initialize AvSynchronizer and create decoder channel
}

// Function     : CAvSyncDecodeWrapper::~CAvSyncDecodeWrapper
// Description  : Destructor
// Parameter    : N/A
// Return value : N/A
//
CAvSyncDecodeWrapper::~CAvSyncDecodeWrapper()
{
	ReleaseModule();  // Delete decoder channel and release AvSynchronizer
}

// Function     : CAvSyncDecodeWrapper::InitialModule
// Description  : Initialize AvSynchronizer and create decoder channel
// Parameter    : N/A
// Return value :
//              | S_OK   @Success
//              | S_FAIL @Fail
//
SCODE CAvSyncDecodeWrapper::InitialModule()
{	
	SCODE scRet;

	// STEP 1: Initialize AvSynchronizer
	scRet = AvSynchronizer_InitialEx(&m_hAvSync, NULL, NULL, NULL, 0, 0, AVSYNCHRONIZER_VERSION, DECODER_CHANNEL_ONLY, 0);

	if	(IS_FAIL(scRet))
	{
		sprintf_s(m_szErrMsgBuf, sizeof(m_szErrMsgBuf), "AvSynchronizer_InitialEx fail! (ERROR: %X)", scRet);

		MessageBox(NULL, m_szErrMsgBuf, NULL, MB_OK);

		return S_FAIL;
	}

	// STEP 2: Create decode channel
	if	(m_hAvSync != NULL)
	{
		scRet = CreateChannel();

		if	(IS_FAIL(scRet))
		{
			return S_FAIL;
		}
	}

	return S_OK;
}

// Function     : CAvSyncDecodeWrapper::ReleaseModule
// Description  : Delete decoder channel and release AvSynchronizer
// Parameter    : N/A
// Return value :
//              | S_OK   @Success
//              | S_FAIL @Fail
//
SCODE CAvSyncDecodeWrapper::ReleaseModule()
{
	SCODE scRet;
	
	if	(m_hAvSync != NULL)
	{
		// STEP 1: Delete decoder channel
		if	(m_hDecodeChannel != NULL)
		{
			scRet = DeleteChannel();

			if	(IS_FAIL(scRet))
			{
				return S_FAIL;
			}
		}

		// STEP 2: Release AvSynchronizer
		scRet = AvSynchronizer_Release(&m_hAvSync);

		if	(IS_FAIL(scRet))
		{
			sprintf_s(m_szErrMsgBuf, sizeof(m_szErrMsgBuf), "AvSynchronizer_Release fail! (ERROR: %X)", scRet);

			MessageBox(NULL, m_szErrMsgBuf, NULL, MB_OK);

			return S_FAIL;
		}
	}

	return S_OK;
}

// Function     : CAvSyncDecodeWrapper::CreateChannel
// Description  : Create decoder channel
// Parameter    : N/A
// Return value :
//              | S_OK   @Success
//              | S_FAIL @Fail
//
SCODE CAvSyncDecodeWrapper::CreateChannel()
{	
	// STEP 1: Set channel options
	TDECCHOPTION tDecCHOption;

	memset(&tDecCHOption, 0, sizeof(tDecCHOption));

	tDecCHOption.pfDecodeFrame     = m_tParam.pfDecodeFrame;
	tDecCHOption.dwAvDecodeContext = m_tParam.dwDecodeFrameContext;
	tDecCHOption.dwRawDataFormat   = m_dwDecodeFormat;
	
	SCODE scRet;

	// STEP 2: Create decoder channel
	
	// Notice!
	// Here use the "AvSynchronizer_CreateDecoderChannel" to create decoder channel, not "AvSynchronizer_CreateChannel"
	scRet = AvSynchronizer_CreateDecoderChannel(m_hAvSync, &m_hDecodeChannel, &tDecCHOption);

	if	(IS_FAIL(scRet))
	{
		sprintf_s(m_szErrMsgBuf, sizeof(m_szErrMsgBuf), "AvSynchronizer_CreateDecoderChannel fail! (ERROR: %X)", scRet);

		MessageBox(NULL, m_szErrMsgBuf, NULL, MB_OK);

		return S_FAIL;
	}
	
	return S_OK;
}

// Function     : CAvSyncDecodeWrapper::DeleteChannel
// Description  : Delete decoder channel
// Parameter    : N/A
// Return value :
//              | S_OK   @Success
//              | S_FAIL @Fail
//
SCODE CAvSyncDecodeWrapper::DeleteChannel()
{
	SCODE scRet;

	// Delete decoder channel 
	scRet = AvSynchronizer_DeleteDecoderChannel(m_hAvSync, &m_hDecodeChannel);

	if	(IS_FAIL(scRet))
	{
		sprintf_s(m_szErrMsgBuf, sizeof(m_szErrMsgBuf), "AvSynchronizer_DeleteDecoderChannel fail! (ERROR: %X)", scRet);

		MessageBox(NULL, m_szErrMsgBuf, NULL, MB_OK);

		return S_FAIL;
	}

	return S_OK;
}

// Function     : CAvSyncDecodeWrapper::SetDecodeFormat
// Description  : Set decode data format of decoder channel
// Parameter    :
//              | [in] dwFormat @Decode data format
// Return value :
//              | S_OK   @Success
//              | S_FAIL @Fail
//
SCODE CAvSyncDecodeWrapper::SetDecodeFormat(DWORD dwFormat)
{
	m_dwDecodeFormat = dwFormat;
	
	EPIXELFORMAT ePixelFormat = EPIXELFORMAT(dwFormat);

	SCODE scRet;

	scRet = AvSynchronizer_SetChannelOption(m_hDecodeChannel, SETCH_DECODE_DATA_FORMAT, ePixelFormat, 0);

	if	(IS_FAIL(scRet))
	{
		sprintf_s(m_szErrMsgBuf, sizeof(m_szErrMsgBuf), "AvSynchronizer_SetChannelOption fail! (ERROR: %X)", scRet);

		OutputDebugString(m_szErrMsgBuf);

		return S_FAIL;
	}

	return S_OK;
}

// Function     : CAvSyncDecodeWrapper::InputToBeDecodedMediaFrame
// Description  : Input the encoded media frame into channel
// Parameter    :
//              | [in] pMediaDataPacket @Input media data packet info
// Return value :
//              | S_OK   @Success
//              | S_FAIL @Fail
//
SCODE CAvSyncDecodeWrapper::InputToBeDecodedMediaFrame(TMediaDataPacketInfo *pMediaDataPacket)
{
	SCODE scRet;

	if	(m_hDecodeChannel != NULL)
	{
		scRet = AvSynchronizer_InputToBeDecodedMediaFrame(m_hDecodeChannel, pMediaDataPacket);

		if	(IS_FAIL(scRet))
		{
			sprintf_s(m_szErrMsgBuf, sizeof(m_szErrMsgBuf), "AvSynchronizer_InputToBeDecodedMediaFrame fail! (ERROR: %X)", scRet);

			OutputDebugString(m_szErrMsgBuf);

			return S_FAIL;
		}
	}

	return S_OK;
}
